// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

#include <log/logger.h>
#include <log/macros.h>
#include <log/log_dbglevels.h>
#include <asiodns/asiodns_messages.h>

namespace isc {
namespace asiodns {

extern isc::log::Logger logger;

}
}
